/*****************************************************
Chip type               : ATtiny13A
AVR Core Clock frequency: 9,6 MHz
*****************************************************/
#include <tiny13a.h>

#define R_upr 27000 //Ohms
#define R_lwr 2000   //Ohms
#define Ub_max_adc ((110UL*(R_upr+R_lwr))/R_lwr) //=U_max*100, 15.95V*100

#define Rhv_upr 510000 //Ohms
#define Rhv_lwr 1500   //Ohms
#define UHV_max_adc ((110UL*(Rhv_upr+Rhv_lwr))/Rhv_lwr) //=U_max*100, 375V*100

#define uptrs   370UL
#define svl     350UL
#define ltrs	 330UL

#define x0 = (ltrs*0x3ff*100)/(UHV_max_adc)
#define x1 = (svl*0x3ff*100)/(UHV_max_adc)
#define x2 = (uptrs*0x3ff*100)/(UHV_max_adc)  

#define DCAC_on PORTB.3
   
volatile unsigned int tim0,tim1;
volatile unsigned int adc2val, adc1val;
volatile unsigned char hv_en=0, ovrld=0, ovl_fg=0;

inline void gpio_init()
{
 PORTB=0;
 DDRB =0b11101011; 
}
inline void t0_init()
{
TCCR0A=1<<COM0A1|1<<COM0B1|1<<COM0B0|1<<WGM00;
TCCR0B=1<<CS00;    
OCR0A=0;
OCR0B=0xff;    
TIMSK0 =1<<TOIE0;
}
inline void adc_init()
{
 ADMUX   =1<<REFS0|2;
 ADCSRA  =1<<ADEN|5;//div 2exp5>>32
 ADCSRA |=1<<ADSC;
}

interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
  { 
  #define rcntdef 8
  static unsigned char rcnt1=rcntdef, rcnt2=rcntdef;
  static unsigned int tmp1, tmp2;

  if(!rcnt2)
  {
    if(rcnt1) //vbat
    {
      rcnt1--;
      tmp1+=ADCW;
    }
    else
    {
      rcnt1=rcntdef; adc1val= tmp1/(rcntdef); tmp1=0;
    }
    ADMUX &= 0b11111100;
    ADMUX |= 2;
    rcnt2=rcntdef;
  }
  else
  {
      rcnt2--;
      tmp2+=ADCW;

    if(!rcnt2)
    {
    adc2val= tmp2/(rcntdef); tmp2=0;
    ADMUX &= 0b11111100;
    ADMUX |= 1;
    }
  }
  }
   ADCSRA |=1<<ADSC;

   if(hv_en)
   {
     if(adc2val<x1){if(OCR0A<(0xff/2)-2){OCR0A++;}
                    else if(adc2val<x0){if(ovrld<0xff){ovrld++;}else{ovl_fg=1;}}}
     if(adc2val>x2){if(OCR0A>9){OCR0A-=10;}else{OCR0A=0;}}
     else if(adc2val>x1){if(OCR0A){OCR0A--;} if(ovrld){ovrld--;}else{ovl_fg=0;}}
     else {if(ovrld){ovrld--;}else{ovl_fg=0;}}

     OCR0B = 0xff-OCR0A;
	}

    if(tim0)tim0--;
    if(tim1)tim1--;



#asm ("wdr")
}

void start_process();

void main(void)
{  
gpio_init();
adc_init();
t0_init();
start_process();


// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

// Watchdog Timer Prescaler: OSC/2k (16mS)
#pragma optsize-
WDTCR=0x18;
WDTCR=0x08;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif


#asm("sei")

while (1)
      {
        static bit trg=0;
        
        if((adc1val<(0x3ffUL*1080/Ub_max_adc))&&(!trg))
        {trg=1;tim1=0xffff;} 
         
        if(((trg)&&(!tim1)&&(adc1val<(0x3ffUL*1080/Ub_max_adc)))||(adc1val<(0x3ffUL*500/Ub_max_adc)))
        {trg=0;start_process();} 
        else if(!tim1){trg=0;}
        
           
      }
}

void start_process()
{
 hv_en=0;DCAC_on=0;
 #asm ("cli")OCR0A=0;OCR0B=0xff;#asm("sei")
 
 while(adc2val<x1)
 {
   if(adc1val>(0x3ffUL*1080/Ub_max_adc))
   {
    if(!tim0)
     {
      tim0=300; 
      #asm ("cli")
      if(OCR0A<(0xff/2 +2)){OCR0A++;}
      else{OCR0A=0;}
      OCR0B = 0xff-OCR0A; 
      #asm("sei")
     } 
     
   }
   else{#asm ("cli")OCR0A=0;OCR0B=0xff;#asm("sei")} 
 }
 hv_en=1;DCAC_on=1;
}
